@@ -251,7 +251,7 @@ func (e *executor) Cleanup() {
 		return
 	}
 
-	defer os.RemoveAll(e.tempDir)
+	defer func() { _ = os.RemoveAll(e.tempDir) }()
 
 	// nothing to do, as there's no cleanup_script
 	if e.config.CleanupExec == "" {
