@@ -25,6 +25,7 @@ import (
 
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 	"gitlab.com/gitlab-org/gitlab-runner/executors"
+	"gitlab.com/gitlab-org/gitlab-runner/executors/docker/internal/labels"
 	"gitlab.com/gitlab-org/gitlab-runner/executors/docker/internal/networks"
 	"gitlab.com/gitlab-org/gitlab-runner/executors/docker/internal/volumes"
 	"gitlab.com/gitlab-org/gitlab-runner/executors/docker/internal/volumes/parser"
@@ -38,23 +39,29 @@ import (
 )
 
 const (
-	DockerExecutorStagePrepare common.ExecutorStage = "docker_prepare"
-	DockerExecutorStageRun     common.ExecutorStage = "docker_run"
-	DockerExecutorStageCleanup common.ExecutorStage = "docker_cleanup"
+	ExecutorStagePrepare common.ExecutorStage = "docker_prepare"
+	ExecutorStageRun     common.ExecutorStage = "docker_run"
+	ExecutorStageCleanup common.ExecutorStage = "docker_cleanup"
 
-	DockerExecutorStageCreatingBuildVolumes common.ExecutorStage = "docker_creating_build_volumes"
-	DockerExecutorStageCreatingServices     common.ExecutorStage = "docker_creating_services"
-	DockerExecutorStageCreatingUserVolumes  common.ExecutorStage = "docker_creating_user_volumes"
-	DockerExecutorStagePullingImage         common.ExecutorStage = "docker_pulling_image"
+	ExecutorStageCreatingBuildVolumes common.ExecutorStage = "docker_creating_build_volumes"
+	ExecutorStageCreatingServices     common.ExecutorStage = "docker_creating_services"
+	ExecutorStageCreatingUserVolumes  common.ExecutorStage = "docker_creating_user_volumes"
+	ExecutorStagePullingImage         common.ExecutorStage = "docker_pulling_image"
 )
 
-var DockerPrebuiltImagesPaths []string
+var PrebuiltImagesPaths []string
+
+const (
+	labelServiceType = "service"
+	labelWaitType    = "wait"
+)
 
 var neverRestartPolicy = container.RestartPolicy{Name: "no"}
 
-var errVolumesManagerUndefined = errors.New("volumesManager is undefined")
-
-var errNetworksManagerUndefined = errors.New("networksManager is undefined")
+var (
+	errVolumesManagerUndefined  = errors.New("volumesManager is undefined")
+	errNetworksManagerUndefined = errors.New("networksManager is undefined")
+)
 
 type executor struct {
 	executors.AbstractExecutor
@@ -80,6 +87,7 @@ type executor struct {
 
 	volumesManager  volumes.Manager
 	networksManager networks.Manager
+	labeler         labels.Labeler
 
 	networkMode container.NetworkMode
 
@@ -89,10 +97,14 @@ type executor struct {
 func init() {
 	runnerFolder, err := osext.ExecutableFolder()
 	if err != nil {
-		logrus.Errorln("Docker executor: unable to detect gitlab-runner folder, prebuilt image helpers will be loaded from DockerHub.", err)
+		logrus.Errorln(
+			"Docker executor: unable to detect gitlab-runner folder, "+
+				"prebuilt image helpers will be loaded from DockerHub.",
+			err,
+		)
 	}
 
-	DockerPrebuiltImagesPaths = []string{
+	PrebuiltImagesPaths = []string{
 		filepath.Join(runnerFolder, "helper-images"),
 		filepath.Join(runnerFolder, "out/helper-images"),
 	}
@@ -103,7 +115,7 @@ func (e *executor) getServiceVariables() []string {
 }
 
 func (e *executor) pullDockerImage(imageName string, ac *types.AuthConfig) (*types.ImageInspect, error) {
-	e.SetCurrentStage(DockerExecutorStagePullingImage)
+	e.SetCurrentStage(ExecutorStagePullingImage)
 	e.Println("Pulling docker image", imageName, "...")
 
 	ref := imageName
@@ -165,7 +177,12 @@ func (e *executor) getDockerImage(imageName string) (image *types.ImageInspect,
 		}
 	}
 
-	registryInfo := auth.ResolveConfigForImage(imageName, e.Build.GetDockerAuthConfig(), e.Shell().User, e.Build.Credentials)
+	registryInfo := auth.ResolveConfigForImage(
+		imageName,
+		e.Build.GetDockerAuthConfig(),
+		e.Shell().User,
+		e.Build.Credentials,
+	)
 	if registryInfo != nil {
 		e.Println(fmt.Sprintf("Authenticating with credentials from %v", registryInfo.Source))
 		e.Debugln(fmt.Sprintf(
@@ -204,7 +221,7 @@ func (e *executor) loadPrebuiltImage(path, ref, tag string) (*types.ImageInspect
 
 		return nil, fmt.Errorf("cannot load prebuilt image: %s: %w", path, err)
 	}
-	defer file.Close()
+	defer func() { _ = file.Close() }()
 
 	e.Debugln("Loading prebuilt image...")
 
@@ -214,7 +231,7 @@ func (e *executor) loadPrebuiltImage(path, ref, tag string) (*types.ImageInspect
 	}
 	options := types.ImageImportOptions{Tag: tag}
 
-	if err := e.client.ImageImportBlocking(e.Context, source, ref, options); err != nil {
+	if err = e.client.ImageImportBlocking(e.Context, source, ref, options); err != nil {
 		return nil, fmt.Errorf("failed to import image: %w", err)
 	}
 
@@ -231,7 +248,11 @@ func (e *executor) getPrebuiltImage() (*types.ImageInspect, error) {
 	if imageNameFromConfig := e.Config.Docker.HelperImage; imageNameFromConfig != "" {
 		imageNameFromConfig = common.AppVersion.Variables().ExpandValue(imageNameFromConfig)
 
-		e.Debugln("Pull configured helper_image for predefined container instead of import bundled image", imageNameFromConfig, "...")
+		e.Debugln(
+			"Pull configured helper_image for predefined container instead of import bundled image",
+			imageNameFromConfig,
+			"...",
+		)
 
 		return e.getDockerImage(imageNameFromConfig)
 	}
@@ -259,8 +280,11 @@ func (e *executor) getLocalHelperImage() *types.ImageInspect {
 	}
 
 	architecture := e.helperImageInfo.Architecture
-	for _, dockerPrebuiltImagesPath := range DockerPrebuiltImagesPaths {
-		dockerPrebuiltImageFilePath := filepath.Join(dockerPrebuiltImagesPath, "prebuilt-"+architecture+prebuiltImageExtension)
+	for _, dockerPrebuiltImagesPath := range PrebuiltImagesPaths {
+		dockerPrebuiltImageFilePath := filepath.Join(
+			dockerPrebuiltImagesPath,
+			"prebuilt-"+architecture+prebuiltImageExtension,
+		)
 		image, err := e.loadPrebuiltImage(dockerPrebuiltImageFilePath, prebuiltImageName, e.helperImageInfo.Tag)
 		if err != nil {
 			e.Debugln("Failed to load prebuilt image from:", dockerPrebuiltImageFilePath, "error:", err)
@@ -288,26 +312,6 @@ func (e *executor) getBuildImage() (*types.ImageInspect, error) {
 	return image, nil
 }
 
-func (e *executor) getLabels(containerType string, otherLabels ...string) map[string]string {
-	labels := make(map[string]string)
-	labels[dockerLabelPrefix+".job.id"] = strconv.Itoa(e.Build.ID)
-	labels[dockerLabelPrefix+".job.sha"] = e.Build.GitInfo.Sha
-	labels[dockerLabelPrefix+".job.before_sha"] = e.Build.GitInfo.BeforeSha
-	labels[dockerLabelPrefix+".job.ref"] = e.Build.GitInfo.Ref
-	labels[dockerLabelPrefix+".project.id"] = strconv.Itoa(e.Build.JobInfo.ProjectID)
-	labels[dockerLabelPrefix+".pipeline.id"] = e.Build.GetAllVariables().Get("CI_PIPELINE_ID")
-	labels[dockerLabelPrefix+".runner.id"] = e.Build.Runner.ShortDescription()
-	labels[dockerLabelPrefix+".runner.local_id"] = strconv.Itoa(e.Build.RunnerID)
-	labels[dockerLabelPrefix+".type"] = containerType
-	for _, label := range otherLabels {
-		keyValue := strings.SplitN(label, "=", 2)
-		if len(keyValue) == 2 {
-			labels[dockerLabelPrefix+"."+keyValue[0]] = keyValue[1]
-		}
-	}
-	return labels
-}
-
 func fakeContainer(id string, names ...string) *types.Container {
 	return &types.Container{ID: id, Names: names}
 }
@@ -376,8 +380,13 @@ func (e *executor) markImageAsUsed(imageName, imageID string) {
 	}
 }
 
-func (e *executor) createService(serviceIndex int, service, version, image string, serviceDefinition common.Image, linkNames []string) (*types.Container, error) {
-	if len(service) == 0 {
+func (e *executor) createService(
+	serviceIndex int,
+	service, version, image string,
+	serviceDefinition common.Image,
+	linkNames []string,
+) (*types.Container, error) {
+	if service == "" {
 		return nil, fmt.Errorf("invalid service name: %s", serviceDefinition.Name)
 	}
 
@@ -391,15 +400,21 @@ func (e *executor) createService(serviceIndex int, service, version, image strin
 		return nil, err
 	}
 
-	serviceSlug := strings.Replace(service, "/", "__", -1)
+	serviceSlug := strings.ReplaceAll(service, "/", "__")
 	containerName := fmt.Sprintf("%s-%s-%d", e.getProjectUniqRandomizedName(), serviceSlug, serviceIndex)
 
 	// this will fail potentially some builds if there's name collision
-	e.removeContainer(e.Context, containerName)
+	_ = e.removeContainer(e.Context, containerName)
+
+	labels := map[string]string{
+		"type":            labelServiceType,
+		"service":         service,
+		"service.version": version,
+	}
 
 	config := &container.Config{
 		Image:  serviceImage.ID,
-		Labels: e.getLabels("service", "service="+service, "service.version="+version),
+		Labels: e.labeler.Labels(labels),
 		Env:    append(e.getServiceVariables(), e.BuildShell.Environment...),
 	}
 
@@ -408,21 +423,7 @@ func (e *executor) createService(serviceIndex int, service, version, image strin
 	}
 	config.Entrypoint = e.overwriteEntrypoint(&serviceDefinition)
 
-	hostConfig := &container.HostConfig{
-		DNS:           e.Config.Docker.DNS,
-		DNSSearch:     e.Config.Docker.DNSSearch,
-		RestartPolicy: neverRestartPolicy,
-		ExtraHosts:    e.Config.Docker.ExtraHosts,
-		Privileged:    e.Config.Docker.Privileged,
-		NetworkMode:   e.networkMode,
-		Binds:         e.volumesManager.Binds(),
-		ShmSize:       e.Config.Docker.ShmSize,
-		Tmpfs:         e.Config.Docker.ServicesTmpfs,
-		LogConfig: container.LogConfig{
-			Type: "json-file",
-		},
-	}
-
+	hostConfig := e.createHostConfigForService()
 	networkConfig := e.networkConfig(linkNames)
 
 	e.Debugln("Creating service container", containerName, "...")
@@ -441,6 +442,23 @@ func (e *executor) createService(serviceIndex int, service, version, image strin
 	return fakeContainer(resp.ID, containerName), nil
 }
 
+func (e *executor) createHostConfigForService() *container.HostConfig {
+	return &container.HostConfig{
+		DNS:           e.Config.Docker.DNS,
+		DNSSearch:     e.Config.Docker.DNSSearch,
+		RestartPolicy: neverRestartPolicy,
+		ExtraHosts:    e.Config.Docker.ExtraHosts,
+		Privileged:    e.Config.Docker.Privileged,
+		NetworkMode:   e.networkMode,
+		Binds:         e.volumesManager.Binds(),
+		ShmSize:       e.Config.Docker.ShmSize,
+		Tmpfs:         e.Config.Docker.ServicesTmpfs,
+		LogConfig: container.LogConfig{
+			Type: "json-file",
+		},
+	}
+}
+
 func (e *executor) networkConfig(aliases []string) *network.NetworkingConfig {
 	if e.networkMode.UserDefined() == "" {
 		return &network.NetworkingConfig{}
@@ -498,7 +516,7 @@ func (e *executor) waitForServices() {
 		for _, service := range e.services {
 			wg.Add(1)
 			go func(service *types.Container) {
-				e.waitForServiceContainer(service, time.Duration(waitForServicesTimeout)*time.Second)
+				_ = e.waitForServiceContainer(service, time.Duration(waitForServicesTimeout)*time.Second)
 				wg.Done()
 			}(service)
 		}
@@ -519,7 +537,11 @@ func (e *executor) buildServiceLinks(linksMap map[string]*types.Container) (link
 	return
 }
 
-func (e *executor) createFromServiceDefinition(serviceIndex int, serviceDefinition common.Image, linksMap map[string]*types.Container) (err error) {
+func (e *executor) createFromServiceDefinition(
+	serviceIndex int,
+	serviceDefinition common.Image,
+	linksMap map[string]*types.Container,
+) error {
 	var container *types.Container
 
 	serviceMeta := services.SplitNameAndVersion(serviceDefinition.Name)
@@ -536,9 +558,17 @@ func (e *executor) createFromServiceDefinition(serviceIndex int, serviceDefiniti
 
 		// Create service if not yet created
 		if container == nil {
-			container, err = e.createService(serviceIndex, serviceMeta.Service, serviceMeta.Version, serviceMeta.ImageName, serviceDefinition, serviceMeta.Aliases)
+			var err error
+			container, err = e.createService(
+				serviceIndex,
+				serviceMeta.Service,
+				serviceMeta.Version,
+				serviceMeta.ImageName,
+				serviceDefinition,
+				serviceMeta.Aliases,
+			)
 			if err != nil {
-				return
+				return err
 			}
 
 			e.Debugln("Created service", serviceDefinition.Name, "as", container.ID)
@@ -547,7 +577,7 @@ func (e *executor) createFromServiceDefinition(serviceIndex int, serviceDefiniti
 		}
 		linksMap[linkName] = container
 	}
-	return
+	return nil
 }
 
 func (e *executor) createBuildNetwork() error {
@@ -592,7 +622,7 @@ func (e *executor) cleanupNetwork(ctx context.Context) error {
 }
 
 func (e *executor) createServices() (err error) {
-	e.SetCurrentStage(DockerExecutorStageCreatingServices)
+	e.SetCurrentStage(ExecutorStageCreatingServices)
 	e.Debugln("Creating services...")
 
 	servicesDefinitions, err := e.getServicesDefinitions()
@@ -619,7 +649,12 @@ func (e *executor) createServices() (err error) {
 	return
 }
 
-func (e *executor) createContainer(containerType string, imageDefinition common.Image, cmd []string, allowedInternalImages []string) (*types.ContainerJSON, error) {
+func (e *executor) createContainer(
+	containerType string,
+	imageDefinition common.Image,
+	cmd []string,
+	allowedInternalImages []string,
+) (*types.ContainerJSON, error) {
 	if e.volumesManager == nil {
 		return nil, errVolumesManagerUndefined
 	}
@@ -636,14 +671,53 @@ func (e *executor) createContainer(containerType string, imageDefinition common.
 
 	// Always create unique, but sequential name
 	containerIndex := len(e.builds)
-	containerName := e.getProjectUniqRandomizedName() + "-" +
-		containerType + "-" + strconv.Itoa(containerIndex)
+	containerName := e.getProjectUniqRandomizedName() + "-" + containerType + "-" + strconv.Itoa(containerIndex)
 
+	config := e.createContainerConfig(containerType, imageDefinition, image.ID, hostname, cmd)
+
+	hostConfig, err := e.createHostConfig()
+	if err != nil {
+		return nil, err
+	}
+
+	aliases := []string{"build", containerName}
+	networkConfig := e.networkConfig(aliases)
+
+	// this will fail potentially some builds if there's name collision
+	_ = e.removeContainer(e.Context, containerName)
+
+	e.Debugln("Creating container", containerName, "...")
+	resp, err := e.client.ContainerCreate(e.Context, config, hostConfig, networkConfig, containerName)
+	if err != nil {
+		if resp.ID != "" {
+			e.temporary = append(e.temporary, resp.ID)
+		}
+		return nil, err
+	}
+
+	inspect, err := e.client.ContainerInspect(e.Context, resp.ID)
+	if err != nil {
+		e.temporary = append(e.temporary, resp.ID)
+		return nil, err
+	}
+
+	e.builds = append(e.builds, resp.ID)
+	e.temporary = append(e.temporary, resp.ID)
+	return &inspect, nil
+}
+
+func (e *executor) createContainerConfig(
+	containerType string,
+	imageDefinition common.Image,
+	imageID string,
+	hostname string,
+	cmd []string,
+) *container.Config {
 	config := &container.Config{
-		Image:        image.ID,
+		Image:        imageID,
 		Hostname:     hostname,
 		Cmd:          cmd,
-		Labels:       e.getLabels(containerType),
+		Labels:       e.labeler.Labels(map[string]string{"type": containerType}),
 		Tty:          false,
 		AttachStdin:  true,
 		AttachStdout: true,
@@ -652,15 +726,18 @@ func (e *executor) createContainer(containerType string, imageDefinition common.
 		StdinOnce:    true,
 		Env:          append(e.Build.GetAllVariables().StringList(), e.BuildShell.Environment...),
 	}
-
 	config.Entrypoint = e.overwriteEntrypoint(&imageDefinition)
 
+	return config
+}
+
+func (e *executor) createHostConfig() (*container.HostConfig, error) {
 	nanoCPUs, err := e.Config.Docker.GetNanoCPUs()
 	if err != nil {
 		return nil, err
 	}
 
-	hostConfig := &container.HostConfig{
+	return &container.HostConfig{
 		Resources: container.Resources{
 			Memory:            e.Config.Docker.GetMemory(),
 			MemorySwap:        e.Config.Docker.GetMemorySwap(),
@@ -693,35 +770,10 @@ func (e *executor) createContainer(containerType string, imageDefinition common.
 		},
 		Tmpfs:   e.Config.Docker.Tmpfs,
 		Sysctls: e.Config.Docker.SysCtls,
-	}
-
-	aliases := []string{"build", containerName}
-	networkConfig := e.networkConfig(aliases)
-
-	// this will fail potentially some builds if there's name collision
-	e.removeContainer(e.Context, containerName)
-
-	e.Debugln("Creating container", containerName, "...")
-	resp, err := e.client.ContainerCreate(e.Context, config, hostConfig, networkConfig, containerName)
-	if err != nil {
-		if resp.ID != "" {
-			e.temporary = append(e.temporary, resp.ID)
-		}
-		return nil, err
-	}
-
-	inspect, err := e.client.ContainerInspect(e.Context, resp.ID)
-	if err != nil {
-		e.temporary = append(e.temporary, resp.ID)
-		return nil, err
-	}
-
-	e.builds = append(e.builds, resp.ID)
-	e.temporary = append(e.temporary, resp.ID)
-	return &inspect, nil
+	}, nil
 }
 
-func (e *executor) watchContainer(ctx context.Context, id string, input io.Reader) error {
+func (e *executor) startAndWatchContainer(ctx context.Context, id string, input io.Reader) error {
 	options := types.ContainerAttachOptions{
 		Stream: true,
 		Stdin:  true,
@@ -745,17 +797,17 @@ func (e *executor) watchContainer(ctx context.Context, id string, input io.Reade
 	// Copy any output to the build trace
 	stdoutErrCh := make(chan error)
 	go func() {
-		_, err := stdcopy.StdCopy(e.Trace, e.Trace, hijacked.Reader)
-		stdoutErrCh <- err
+		_, errCopy := stdcopy.StdCopy(e.Trace, e.Trace, hijacked.Reader)
+		stdoutErrCh <- errCopy
 	}()
 
 	// Write the input to the container and close its STDIN to get it to finish
 	stdinErrCh := make(chan error)
 	go func() {
-		_, err := io.Copy(hijacked.Conn, input)
-		hijacked.CloseWrite()
-		if err != nil {
-			stdinErrCh <- err
+		_, errCopy := io.Copy(hijacked.Conn, input)
+		_ = hijacked.CloseWrite()
+		if errCopy != nil {
+			stdinErrCh <- errCopy
 		}
 	}()
 
@@ -814,9 +866,21 @@ func (e *executor) disconnectNetwork(ctx context.Context, id string) {
 			if id == pluggedContainer.Name {
 				err = e.client.NetworkDisconnect(ctx, network.ID, id, true)
 				if err != nil {
-					e.Warningln("Can't disconnect possibly zombie container", pluggedContainer.Name, "from network", network.Name, "->", err)
+					e.Warningln(
+						"Can't disconnect possibly zombie container",
+						pluggedContainer.Name,
+						"from network",
+						network.Name,
+						"->",
+						err,
+					)
 				} else {
-					e.Warningln("Possibly zombie container", pluggedContainer.Name, "is disconnected from network", network.Name)
+					e.Warningln(
+						"Possibly zombie container",
+						pluggedContainer.Name,
+						"is disconnected from network",
+						network.Name,
+					)
 				}
 				break
 			}
@@ -824,7 +888,7 @@ func (e *executor) disconnectNetwork(ctx context.Context, id string) {
 	}
 }
 
-func (e *executor) verifyAllowedImage(image, optionName string, allowedImages []string, internalImages []string) error {
+func (e *executor) verifyAllowedImage(image, optionName string, allowedImages, internalImages []string) error {
 	for _, allowedImage := range allowedImages {
 		ok, _ := doublestar.Match(allowedImage, image)
 		if ok {
@@ -850,7 +914,9 @@ func (e *executor) verifyAllowedImage(image, optionName string, allowedImages []
 		return nil
 	}
 
-	e.Println("Please check runner's configuration: http://doc.gitlab.com/ci/docker/using_docker_images.html#overwrite-image-and-services")
+	e.Println(
+		"Please check runner's configuration: " +
+			"http://doc.gitlab.com/ci/docker/using_docker_images.html#overwrite-image-and-services")
 	return errors.New("invalid image")
 }
 
@@ -931,6 +997,7 @@ func (e *executor) validateOSType() error {
 
 func (e *executor) createDependencies() error {
 	createDependenciesStrategy := []func() error{
+		e.createLabeler,
 		e.createNetworksManager,
 		e.createBuildNetwork,
 		e.bindDevices,
@@ -951,7 +1018,7 @@ func (e *executor) createDependencies() error {
 }
 
 func (e *executor) createVolumes() error {
-	e.SetCurrentStage(DockerExecutorStageCreatingUserVolumes)
+	e.SetCurrentStage(ExecutorStageCreatingUserVolumes)
 	e.Debugln("Creating user-defined volumes...")
 
 	if e.volumesManager == nil {
@@ -977,7 +1044,7 @@ func (e *executor) createVolumes() error {
 }
 
 func (e *executor) createBuildVolume() error {
-	e.SetCurrentStage(DockerExecutorStageCreatingBuildVolumes)
+	e.SetCurrentStage(ExecutorStageCreatingBuildVolumes)
 	e.Debugln("Creating build volume...")
 
 	if e.volumesManager == nil {
@@ -1012,7 +1079,7 @@ func (e *executor) createBuildVolume() error {
 }
 
 func (e *executor) Prepare(options common.ExecutorPrepareOptions) error {
-	e.SetCurrentStage(DockerExecutorStagePrepare)
+	e.SetCurrentStage(ExecutorStagePrepare)
 
 	if options.Config.Docker == nil {
 		return errors.New("missing docker configuration")
@@ -1076,7 +1143,7 @@ func (e *executor) prepareBuildsDir(options common.ExecutorPrepareOptions) error
 }
 
 func (e *executor) Cleanup() {
-	e.SetCurrentStage(DockerExecutorStageCleanup)
+	e.SetCurrentStage(ExecutorStageCleanup)
 
 	var wg sync.WaitGroup
 
@@ -1086,7 +1153,7 @@ func (e *executor) Cleanup() {
 	remove := func(id string) {
 		wg.Add(1)
 		go func() {
-			e.removeContainer(ctx, id)
+			_ = e.removeContainer(ctx, id)
 			wg.Done()
 		}()
 	}
@@ -1117,7 +1184,7 @@ func (e *executor) Cleanup() {
 	}
 
 	if e.client != nil {
-		e.client.Close()
+		_ = e.client.Close()
 	}
 
 	e.AbstractExecutor.Cleanup()
@@ -1165,27 +1232,15 @@ func (e *executor) runServiceHealthCheckContainer(service *types.Container, time
 
 	cmd := []string{"gitlab-runner-helper", "health-check"}
 
-	config := &container.Config{
-		Cmd:    cmd,
-		Image:  waitImage.ID,
-		Labels: e.getLabels("wait", "wait="+service.ID),
-		Env:    environment,
-	}
-	hostConfig := &container.HostConfig{
-		RestartPolicy: neverRestartPolicy,
-		Links:         []string{service.Names[0] + ":service"},
-		NetworkMode:   e.networkMode,
-		LogConfig: container.LogConfig{
-			Type: "json-file",
-		},
-	}
+	config := e.createConfigForServiceHealthCheckContainer(service, cmd, waitImage, environment)
+	hostConfig := e.createHostConfigForServiceHealthCheck(service)
 
 	e.Debugln(fmt.Sprintf("Creating service healthcheck container %s...", containerName))
 	resp, err := e.client.ContainerCreate(e.Context, config, hostConfig, nil, containerName)
 	if err != nil {
 		return fmt.Errorf("create service container: %w", err)
 	}
-	defer e.removeContainer(e.Context, resp.ID)
+	defer func() { _ = e.removeContainer(e.Context, resp.ID) }()
 
 	e.Debugln(fmt.Sprintf("Starting service healthcheck container %s (%s)...", containerName, resp.ID))
 	err = e.client.ContainerStart(e.Context, resp.ID, types.ContainerStartOptions{})
@@ -1213,6 +1268,31 @@ func (e *executor) runServiceHealthCheckContainer(service *types.Container, time
 	}
 }
 
+func (e *executor) createConfigForServiceHealthCheckContainer(
+	service *types.Container,
+	cmd []string,
+	waitImage *types.ImageInspect,
+	environment []string,
+) *container.Config {
+	return &container.Config{
+		Cmd:    cmd,
+		Image:  waitImage.ID,
+		Labels: e.labeler.Labels(map[string]string{"type": labelWaitType, "wait": service.ID}),
+		Env:    environment,
+	}
+}
+
+func (e *executor) createHostConfigForServiceHealthCheck(service *types.Container) *container.HostConfig {
+	return &container.HostConfig{
+		RestartPolicy: neverRestartPolicy,
+		Links:         []string{service.Names[0] + ":service"},
+		NetworkMode:   e.networkMode,
+		LogConfig: container.LogConfig{
+			Type: "json-file",
+		},
+	}
+}
+
 // addServiceHealthCheckEnvironment will create the environment variables needed
 // for our healthcheck system if a network per build was created, until we
 // introduce the native HEALTHCHECK from docker in
@@ -1263,7 +1343,9 @@ func (e *executor) waitForServiceContainer(service *types.Container, timeout tim
 
 	var buffer bytes.Buffer
 	buffer.WriteString("\n")
-	buffer.WriteString(helpers.ANSI_YELLOW + "*** WARNING:" + helpers.ANSI_RESET + " Service " + service.Names[0] + " probably didn't start properly.\n")
+	buffer.WriteString(
+		helpers.ANSI_YELLOW + "*** WARNING:" + helpers.ANSI_RESET + " Service " + service.Names[0] +
+			" probably didn't start properly.\n")
 	buffer.WriteString("\n")
 	buffer.WriteString("Health check error:\n")
 	buffer.WriteString(strings.TrimSpace(err.Error()))
@@ -1284,7 +1366,7 @@ func (e *executor) waitForServiceContainer(service *types.Container, timeout tim
 	buffer.WriteString("\n")
 	buffer.WriteString(helpers.ANSI_YELLOW + "*********" + helpers.ANSI_RESET + "\n")
 	buffer.WriteString("\n")
-	io.Copy(e.Trace, &buffer)
+	_, _ = io.Copy(e.Trace, &buffer)
 	return err
 }
 
@@ -1301,9 +1383,9 @@ func (e *executor) readContainerLogs(containerID string) string {
 	if err != nil {
 		return strings.TrimSpace(err.Error())
 	}
-	defer hijacked.Close()
+	defer func() { _ = hijacked.Close() }()
 
-	stdcopy.StdCopy(&containerBuffer, &containerBuffer, hijacked)
+	_, _ = stdcopy.StdCopy(&containerBuffer, &containerBuffer, hijacked)
 	containerLog := containerBuffer.String()
 	return strings.TrimSpace(containerLog)
 }
