@@ -34,10 +34,11 @@ import (
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/docker"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/docker/auth"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/featureflags"
+	"gitlab.com/gitlab-org/gitlab-runner/helpers/test"
 )
 
 func TestMain(m *testing.M) {
-	DockerPrebuiltImagesPaths = []string{"../../out/helper-images/"}
+	PrebuiltImagesPaths = []string{"../../out/helper-images/"}
 
 	flag.Parse()
 	os.Exit(m.Run())
@@ -45,7 +46,7 @@ func TestMain(m *testing.M) {
 
 // ImagePullOptions contains the RegistryAuth which is inferred from the docker
 // configuration for the user, so just mock it out here.
-func buildImagePullOptions(e *executor, configName string) mock.AnythingOfTypeArgument {
+func buildImagePullOptions() mock.AnythingOfTypeArgument {
 	return mock.AnythingOfType("ImagePullOptions")
 }
 
@@ -149,7 +150,7 @@ func testServiceFromNamedImage(t *testing.T, description, imageName, serviceName
 	c := new(docker.MockClient)
 	defer c.AssertExpectations(t)
 
-	servicePart := fmt.Sprintf("-%s-0", strings.Replace(serviceName, "/", "__", -1))
+	servicePart := fmt.Sprintf("-%s-0", strings.ReplaceAll(serviceName, "/", "__"))
 	containerNameRegex, err := regexp.Compile("runner-abcdef12-project-0-concurrent-0-[^-]+" + servicePart)
 	require.NoError(t, err)
 
@@ -165,7 +166,7 @@ func testServiceFromNamedImage(t *testing.T, description, imageName, serviceName
 		volumeParser: parser.NewLinuxParser(),
 	}
 
-	options := buildImagePullOptions(e, imageName)
+	options := buildImagePullOptions()
 	e.Config = common.RunnerConfig{}
 	e.Config.Docker = &common.DockerConfig{}
 	e.Build = &common.Build{
@@ -183,6 +184,9 @@ func testServiceFromNamedImage(t *testing.T, description, imageName, serviceName
 	})
 	require.NoError(t, err)
 
+	err = e.createLabeler()
+	require.NoError(t, err)
+
 	e.BuildShell = &common.ShellConfiguration{
 		Environment: []string{},
 	}
@@ -197,7 +201,12 @@ func testServiceFromNamedImage(t *testing.T, description, imageName, serviceName
 		Return(types.ImageInspect{ID: "image-id"}, nil, nil).
 		Twice()
 
-	c.On("ContainerRemove", e.Context, containerNameMatcher, types.ContainerRemoveOptions{RemoveVolumes: true, Force: true}).
+	c.On(
+		"ContainerRemove",
+		e.Context,
+		containerNameMatcher,
+		types.ContainerRemoveOptions{RemoveVolumes: true, Force: true},
+	).
 		Return(nil).
 		Once()
 
@@ -243,7 +252,7 @@ func TestDockerForNamedImage(t *testing.T) {
 	validSHA := "real@sha256:b5bb9d8014a0f9b1d61e21e796d78dccdf1352f23cd32812f4850b878ae4944c"
 
 	e := executorWithMockClient(c)
-	options := buildImagePullOptions(e, "test")
+	options := buildImagePullOptions()
 
 	c.On("ImagePullBlocking", e.Context, "test:latest", options).
 		Return(os.ErrNotExist).
@@ -275,7 +284,7 @@ func TestDockerForExistingImage(t *testing.T) {
 	defer c.AssertExpectations(t)
 
 	e := executorWithMockClient(c)
-	options := buildImagePullOptions(e, "existing")
+	options := buildImagePullOptions()
 
 	c.On("ImagePullBlocking", e.Context, "existing:latest", options).
 		Return(nil).
@@ -413,7 +422,7 @@ func TestDockerPolicyModeIfNotPresentForNotExistingImage(t *testing.T) {
 		Return(types.ImageInspect{}, nil, os.ErrNotExist).
 		Once()
 
-	options := buildImagePullOptions(e, "not-existing")
+	options := buildImagePullOptions()
 	c.On("ImagePullBlocking", e.Context, "not-existing:latest", options).
 		Return(nil).
 		Once()
@@ -447,7 +456,7 @@ func TestDockerPolicyModeAlwaysForExistingImage(t *testing.T) {
 		Return(types.ImageInspect{ID: "image-id"}, nil, nil).
 		Once()
 
-	options := buildImagePullOptions(e, "existing:latest")
+	options := buildImagePullOptions()
 	c.On("ImagePullBlocking", e.Context, "existing:latest", options).
 		Return(nil).
 		Once()
@@ -472,7 +481,7 @@ func TestDockerPolicyModeAlwaysForLocalOnlyImage(t *testing.T) {
 		Return(types.ImageInspect{ID: "image-id"}, nil, nil).
 		Once()
 
-	options := buildImagePullOptions(e, "existing:lastest")
+	options := buildImagePullOptions()
 	c.On("ImagePullBlocking", e.Context, "existing:latest", options).
 		Return(fmt.Errorf("not found")).
 		Once()
@@ -493,7 +502,7 @@ func TestDockerGetExistingDockerImageIfPullFails(t *testing.T) {
 		Return(types.ImageInspect{ID: "image-id"}, nil, nil).
 		Once()
 
-	options := buildImagePullOptions(e, "to-pull")
+	options := buildImagePullOptions()
 	c.On("ImagePullBlocking", e.Context, "to-pull:latest", options).
 		Return(os.ErrNotExist).
 		Once()
@@ -1040,7 +1049,14 @@ func TestCreateDependencies(t *testing.T) {
 			c.On("ContainerRemove", mock.Anything, containerID, mock.Anything).
 				Return(nil).
 				Once()
-			c.On("ContainerCreate", mock.Anything, mock.Anything, hostConfigMatcher, mock.Anything, containerNameMatcher).
+			c.On(
+				"ContainerCreate",
+				mock.Anything,
+				mock.Anything,
+				hostConfigMatcher,
+				mock.Anything,
+				containerNameMatcher,
+			).
 				Return(container.ContainerCreateCreatedBody{ID: containerID}, nil).
 				Once()
 			c.On("ContainerStart", mock.Anything, containerID, mock.Anything).
@@ -1056,7 +1072,7 @@ func TestCreateDependencies(t *testing.T) {
 	assert.Equal(t, testError, err)
 }
 
-func getTestExecutor(t *testing.T) *executor {
+func getTestExecutor() *executor {
 	e := new(executor)
 	e.Build = &common.Build{
 		Runner: &common.RunnerConfig{},
@@ -1072,7 +1088,12 @@ func getTestExecutor(t *testing.T) *executor {
 	return e
 }
 
-func testGetDockerImage(t *testing.T, e *executor, imageName string, setClientExpectations func(c *docker.MockClient, imageName string)) {
+func testGetDockerImage(
+	t *testing.T,
+	e *executor,
+	imageName string,
+	setClientExpectations func(c *docker.MockClient, imageName string),
+) {
 	t.Run("get:"+imageName, func(t *testing.T) {
 		c := new(docker.MockClient)
 		defer c.AssertExpectations(t)
@@ -1087,7 +1108,12 @@ func testGetDockerImage(t *testing.T, e *executor, imageName string, setClientEx
 	})
 }
 
-func testDeniesDockerImage(t *testing.T, e *executor, imageName string, setClientExpectations func(c *docker.MockClient, imageName string)) {
+func testDeniesDockerImage(
+	t *testing.T,
+	e *executor,
+	imageName string,
+	setClientExpectations func(c *docker.MockClient, imageName string),
+) {
 	t.Run("deny:"+imageName, func(t *testing.T) {
 		c := new(docker.MockClient)
 		defer c.AssertExpectations(t)
@@ -1135,7 +1161,7 @@ func TestPullPolicyWhenAlwaysIsSet(t *testing.T) {
 	remoteImage := "registry.domain.tld:5005/image/name:version"
 	gitlabImage := "registry.gitlab.tld:1234/image/name:version"
 
-	e := getTestExecutor(t)
+	e := getTestExecutor()
 	e.Context = context.Background()
 	e.Config.Docker.PullPolicy = common.PullPolicyAlways
 
@@ -1150,7 +1176,7 @@ func TestPullPolicyWhenIfNotPresentIsSet(t *testing.T) {
 	remoteImage := "registry.domain.tld:5005/image/name:version"
 	gitlabImage := "registry.gitlab.tld:1234/image/name:version"
 
-	e := getTestExecutor(t)
+	e := getTestExecutor()
 	e.Context = context.Background()
 	e.Config.Docker.PullPolicy = common.PullPolicyIfNotPresent
 
@@ -1159,6 +1185,7 @@ func TestPullPolicyWhenIfNotPresentIsSet(t *testing.T) {
 }
 
 func TestDockerWatchOn_1_12_4(t *testing.T) {
+	test.SkipIfGitLabCIOn(t, test.OSWindows)
 	if helpers.SkipIntegrationTests(t, "docker", "info") {
 		return
 	}
@@ -1196,7 +1223,15 @@ func TestDockerWatchOn_1_12_4(t *testing.T) {
 	err = e.createVolumesManager()
 	require.NoError(t, err)
 
-	container, err := e.createContainer("build", common.Image{Name: common.TestAlpineImage}, []string{"/bin/sh"}, []string{})
+	err = e.createLabeler()
+	require.NoError(t, err)
+
+	container, err := e.createContainer(
+		"build",
+		common.Image{Name: common.TestAlpineImage},
+		[]string{"/bin/sh"},
+		[]string{},
+	)
 	assert.NoError(t, err)
 	assert.NotNil(t, container)
 
@@ -1206,7 +1241,7 @@ func TestDockerWatchOn_1_12_4(t *testing.T) {
 	wg := &sync.WaitGroup{}
 	wg.Add(1) // Avoid a race where assert.NoError() is called too late in the goroutine
 	go func() {
-		err = e.watchContainer(e.Context, container.ID, input)
+		err = e.startAndWatchContainer(e.Context, container.ID, input)
 		assert.NoError(t, err)
 		finished <- true
 		wg.Done()
@@ -1233,12 +1268,22 @@ type dockerConfigurationTestFakeDockerClient struct {
 	t   *testing.T
 }
 
-func (c *dockerConfigurationTestFakeDockerClient) ContainerCreate(ctx context.Context, config *container.Config, hostConfig *container.HostConfig, networkingConfig *network.NetworkingConfig, containerName string) (container.ContainerCreateCreatedBody, error) {
+func (c *dockerConfigurationTestFakeDockerClient) ContainerCreate(
+	ctx context.Context,
+	config *container.Config,
+	hostConfig *container.HostConfig,
+	networkingConfig *network.NetworkingConfig,
+	containerName string,
+) (container.ContainerCreateCreatedBody, error) {
 	c.cce(c.t, config, hostConfig)
 	return container.ContainerCreateCreatedBody{ID: "abc"}, nil
 }
 
-func prepareTestDockerConfiguration(t *testing.T, dockerConfig *common.DockerConfig, cce containerConfigExpectations) (*dockerConfigurationTestFakeDockerClient, *executor) {
+func prepareTestDockerConfiguration(
+	t *testing.T,
+	dockerConfig *common.DockerConfig,
+	cce containerConfigExpectations,
+) (*dockerConfigurationTestFakeDockerClient, *executor) {
 	c := &dockerConfigurationTestFakeDockerClient{
 		cce: cce,
 		t:   t,
@@ -1267,6 +1312,9 @@ func prepareTestDockerConfiguration(t *testing.T, dockerConfig *common.DockerCon
 	})
 	require.NoError(t, err)
 
+	err = e.createLabeler()
+	require.NoError(t, err)
+
 	c.On("ImageInspectWithRaw", mock.Anything, "alpine").
 		Return(types.ImageInspect{ID: "123"}, []byte{}, nil).Twice()
 	c.On("ImagePullBlocking", mock.Anything, "alpine:latest", mock.Anything).
@@ -1279,7 +1327,11 @@ func prepareTestDockerConfiguration(t *testing.T, dockerConfig *common.DockerCon
 	return c, e
 }
 
-func testDockerConfigurationWithJobContainer(t *testing.T, dockerConfig *common.DockerConfig, cce containerConfigExpectations) {
+func testDockerConfigurationWithJobContainer(
+	t *testing.T,
+	dockerConfig *common.DockerConfig,
+	cce containerConfigExpectations,
+) {
 	c, e := prepareTestDockerConfiguration(t, dockerConfig, cce)
 	defer c.AssertExpectations(t)
 
@@ -1293,7 +1345,11 @@ func testDockerConfigurationWithJobContainer(t *testing.T, dockerConfig *common.
 	assert.NoError(t, err, "Should create container without errors")
 }
 
-func testDockerConfigurationWithServiceContainer(t *testing.T, dockerConfig *common.DockerConfig, cce containerConfigExpectations) {
+func testDockerConfigurationWithServiceContainer(
+	t *testing.T,
+	dockerConfig *common.DockerConfig,
+	cce containerConfigExpectations,
+) {
 	c, e := prepareTestDockerConfiguration(t, dockerConfig, cce)
 	defer c.AssertExpectations(t)
 
@@ -1303,7 +1359,14 @@ func testDockerConfigurationWithServiceContainer(t *testing.T, dockerConfig *com
 	err := e.createVolumesManager()
 	require.NoError(t, err)
 
-	_, err = e.createService(0, "build", "latest", "alpine", common.Image{Command: []string{"/bin/sh"}}, nil)
+	_, err = e.createService(
+		0,
+		"build",
+		"latest",
+		"alpine",
+		common.Image{Command: []string{"/bin/sh"}},
+		nil,
+	)
 	assert.NoError(t, err, "Should create service container without errors")
 }
 
@@ -1720,7 +1783,7 @@ func TestCheckOSType(t *testing.T) {
 
 	for name, c := range cases {
 		t.Run(name, func(t *testing.T) {
-			executor := executor{
+			e := executor{
 				info: types.Info{
 					OSType: c.dockerInfoOSType,
 				},
@@ -1731,7 +1794,7 @@ func TestCheckOSType(t *testing.T) {
 				},
 			}
 
-			err := executor.validateOSType()
+			err := e.validateOSType()
 			if c.expectedErr == "" {
 				assert.NoError(t, err)
 				return
