@@ -120,24 +120,24 @@ func (l *kubernetesLogProcessor) Process(ctx context.Context) <-chan string {
 
 func (l *kubernetesLogProcessor) attach(ctx context.Context, outCh chan string) {
 	var attempt float64 = -1
+	var backoffDuration time.Duration
 
 	for {
-		select {
-		case <-ctx.Done():
-			return
-		default:
-		}
-
 		attempt++
 		if attempt > 0 {
-			backoffDuration := l.backoff.ForAttempt(attempt)
+			backoffDuration = l.backoff.ForAttempt(attempt)
 			l.logger.Debugln(fmt.Sprintf("Backing off reattaching log for %s for %s", l.logStreamer, backoffDuration))
-			time.Sleep(backoffDuration)
 		}
 
-		err := l.processStream(ctx, outCh)
-		if err != nil {
-			l.logger.Warningln(fmt.Sprintf("Error %s. Retrying...", err))
+		select {
+		case <-ctx.Done():
+			l.logger.Debugln(fmt.Sprintf("Detaching from log... %v", ctx.Err()))
+			return
+		case <-time.After(backoffDuration):
+			err := l.processStream(ctx, outCh)
+			if err != nil {
+				l.logger.Warningln(fmt.Sprintf("Error %v. Retrying...", err))
+			}
 		}
 	}
 }
@@ -234,7 +234,7 @@ func (l *kubernetesLogProcessor) scan(ctx context.Context, logs io.Reader) (*buf
 // The line doesn't include the new line character.
 // Lines without offset are acceptable and return -1 for offset.
 func (l *kubernetesLogProcessor) parseLogLine(line string) (int64, string) {
-	if len(line) == 0 {
+	if line == "" {
 		return -1, ""
 	}
 
