@@ -14,17 +14,15 @@ import (
 )
 
 func TestWriteGitSSLConfig(t *testing.T) {
-	gitlabURL := "https://example.com:3443"
-	runnerURL := gitlabURL + "/ci/"
+	expectedURL := "https://example.com:3443"
 
 	shell := AbstractShell{}
 	build := &common.Build{
-		Runner: &common.RunnerConfig{
-			RunnerCredentials: common.RunnerCredentials{
-				URL: runnerURL,
-			},
-		},
+		Runner: &common.RunnerConfig{},
 		JobResponse: common.JobResponse{
+			GitInfo: common.GitInfo{
+				RepoURL: "https://gitlab-ci-token:xxx@example.com:3443/project/repo.git",
+			},
 			TLSAuthCert: "TLS_CERT",
 			TLSAuthKey:  "TLS_KEY",
 			TLSCAChain:  "CA_CHAIN",
@@ -36,16 +34,34 @@ func TestWriteGitSSLConfig(t *testing.T) {
 	mockWriter.On("EnvVariableKey", tls.VariableCertFile).Return("VariableCertFile").Once()
 	mockWriter.On("EnvVariableKey", tls.VariableKeyFile).Return("VariableKeyFile").Once()
 
-	mockWriter.On("Command", "git", "config", fmt.Sprintf("http.%s.%s", gitlabURL, "sslCAInfo"), "VariableCAFile").Once()
-	mockWriter.On("Command", "git", "config", fmt.Sprintf("http.%s.%s", gitlabURL, "sslCert"), "VariableCertFile").Once()
-	mockWriter.On("Command", "git", "config", fmt.Sprintf("http.%s.%s", gitlabURL, "sslKey"), "VariableKeyFile").Once()
+	mockWriter.On(
+		"Command",
+		"git",
+		"config",
+		fmt.Sprintf("http.%s.%s", expectedURL, "sslCAInfo"),
+		"VariableCAFile",
+	).Once()
+	mockWriter.On(
+		"Command",
+		"git",
+		"config",
+		fmt.Sprintf("http.%s.%s", expectedURL, "sslCert"),
+		"VariableCertFile",
+	).Once()
+	mockWriter.On(
+		"Command",
+		"git",
+		"config",
+		fmt.Sprintf("http.%s.%s", expectedURL, "sslKey"),
+		"VariableKeyFile",
+	).Once()
 
 	shell.writeGitSSLConfig(mockWriter, build, nil)
 
 	mockWriter.AssertExpectations(t)
 }
 
-func getJobResponseWithMultipleArtifacts(t *testing.T) common.JobResponse {
+func getJobResponseWithMultipleArtifacts() common.JobResponse {
 	return common.JobResponse{
 		ID:    1000,
 		Token: "token",
@@ -86,7 +102,7 @@ func TestWriteWritingArtifactsOnSuccess(t *testing.T) {
 
 	shell := AbstractShell{}
 	build := &common.Build{
-		JobResponse: getJobResponseWithMultipleArtifacts(t),
+		JobResponse: getJobResponseWithMultipleArtifacts(),
 		Runner: &common.RunnerConfig{
 			RunnerCredentials: common.RunnerCredentials{
 				URL: gitlabURL,
@@ -103,38 +119,48 @@ func TestWriteWritingArtifactsOnSuccess(t *testing.T) {
 	mockWriter.On("Variable", mock.Anything)
 	mockWriter.On("Cd", mock.Anything)
 	mockWriter.On("IfCmd", "gitlab-runner-helper", "--version")
-	mockWriter.On("Notice", mock.Anything)
-	mockWriter.On("Command", "gitlab-runner-helper", "artifacts-uploader",
+	mockWriter.On("Noticef", mock.Anything)
+	mockWriter.On(
+		"Command", "gitlab-runner-helper", "artifacts-uploader",
 		"--url", gitlabURL,
 		"--token", "token",
 		"--id", "1000",
-		"--path", "default").Once()
-	mockWriter.On("Command", "gitlab-runner-helper", "artifacts-uploader",
+		"--path", "default",
+	).Once()
+	mockWriter.On(
+		"Command", "gitlab-runner-helper", "artifacts-uploader",
 		"--url", gitlabURL,
 		"--token", "token",
 		"--id", "1000",
-		"--path", "on-success").Once()
-	mockWriter.On("Command", "gitlab-runner-helper", "artifacts-uploader",
+		"--path", "on-success",
+	).Once()
+	mockWriter.On(
+		"Command", "gitlab-runner-helper", "artifacts-uploader",
 		"--url", gitlabURL,
 		"--token", "token",
 		"--id", "1000",
-		"--path", "always").Once()
-	mockWriter.On("Command", "gitlab-runner-helper", "artifacts-uploader",
+		"--path", "always",
+	).Once()
+	mockWriter.On(
+		"Command", "gitlab-runner-helper", "artifacts-uploader",
 		"--url", gitlabURL,
 		"--token", "token",
 		"--id", "1000",
 		"--path", "zip-archive",
 		"--artifact-format", "zip",
-		"--artifact-type", "archive").Once()
-	mockWriter.On("Command", "gitlab-runner-helper", "artifacts-uploader",
+		"--artifact-type", "archive",
+	).Once()
+	mockWriter.On(
+		"Command", "gitlab-runner-helper", "artifacts-uploader",
 		"--url", gitlabURL,
 		"--token", "token",
 		"--id", "1000",
 		"--path", "gzip-junit",
 		"--artifact-format", "gzip",
-		"--artifact-type", "junit").Once()
+		"--artifact-type", "junit",
+	).Once()
 	mockWriter.On("Else")
-	mockWriter.On("Warning", mock.Anything, mock.Anything, mock.Anything)
+	mockWriter.On("Warningf", mock.Anything, mock.Anything, mock.Anything)
 	mockWriter.On("EndIf")
 
 	err := shell.writeScript(mockWriter, common.BuildStageUploadOnSuccessArtifacts, info)
@@ -146,7 +172,7 @@ func TestWriteWritingArtifactsOnFailure(t *testing.T) {
 
 	shell := AbstractShell{}
 	build := &common.Build{
-		JobResponse: getJobResponseWithMultipleArtifacts(t),
+		JobResponse: getJobResponseWithMultipleArtifacts(),
 		Runner: &common.RunnerConfig{
 			RunnerCredentials: common.RunnerCredentials{
 				URL: gitlabURL,
@@ -163,33 +189,41 @@ func TestWriteWritingArtifactsOnFailure(t *testing.T) {
 	mockWriter.On("Variable", mock.Anything)
 	mockWriter.On("Cd", mock.Anything)
 	mockWriter.On("IfCmd", "gitlab-runner-helper", "--version")
-	mockWriter.On("Notice", mock.Anything)
-	mockWriter.On("Command", "gitlab-runner-helper", "artifacts-uploader",
+	mockWriter.On("Noticef", mock.Anything)
+	mockWriter.On(
+		"Command", "gitlab-runner-helper", "artifacts-uploader",
 		"--url", gitlabURL,
 		"--token", "token",
 		"--id", "1000",
-		"--path", "on-failure").Once()
-	mockWriter.On("Command", "gitlab-runner-helper", "artifacts-uploader",
+		"--path", "on-failure",
+	).Once()
+	mockWriter.On(
+		"Command", "gitlab-runner-helper", "artifacts-uploader",
 		"--url", gitlabURL,
 		"--token", "token",
 		"--id", "1000",
-		"--path", "always").Once()
-	mockWriter.On("Command", "gitlab-runner-helper", "artifacts-uploader",
+		"--path", "always",
+	).Once()
+	mockWriter.On(
+		"Command", "gitlab-runner-helper", "artifacts-uploader",
 		"--url", gitlabURL,
 		"--token", "token",
 		"--id", "1000",
 		"--path", "zip-archive",
 		"--artifact-format", "zip",
-		"--artifact-type", "archive").Once()
-	mockWriter.On("Command", "gitlab-runner-helper", "artifacts-uploader",
+		"--artifact-type", "archive",
+	).Once()
+	mockWriter.On(
+		"Command", "gitlab-runner-helper", "artifacts-uploader",
 		"--url", gitlabURL,
 		"--token", "token",
 		"--id", "1000",
 		"--path", "gzip-junit",
 		"--artifact-format", "gzip",
-		"--artifact-type", "junit").Once()
+		"--artifact-type", "junit",
+	).Once()
 	mockWriter.On("Else")
-	mockWriter.On("Warning", mock.Anything, mock.Anything, mock.Anything)
+	mockWriter.On("Warningf", mock.Anything, mock.Anything, mock.Anything)
 	mockWriter.On("EndIf")
 
 	err := shell.writeScript(mockWriter, common.BuildStageUploadOnFailureArtifacts, info)
@@ -230,17 +264,19 @@ func TestWriteWritingArtifactsWithExcludedPaths(t *testing.T) {
 	mockWriter.On("Variable", mock.Anything)
 	mockWriter.On("Cd", mock.Anything).Once()
 	mockWriter.On("IfCmd", "gitlab-runner-helper", "--version").Once()
-	mockWriter.On("Notice", mock.Anything).Once()
-	mockWriter.On("Command", "gitlab-runner-helper", "artifacts-uploader",
+	mockWriter.On("Noticef", mock.Anything).Once()
+	mockWriter.On(
+		"Command", "gitlab-runner-helper", "artifacts-uploader",
 		"--url", "https://gitlab.example.com",
 		"--token", "token",
 		"--id", "1001",
 		"--path", "include/**",
 		"--exclude", "include/exclude/*",
 		"--artifact-format", "zip",
-		"--artifact-type", "archive").Once()
+		"--artifact-type", "archive",
+	).Once()
 	mockWriter.On("Else").Once()
-	mockWriter.On("Warning", mock.Anything, mock.Anything, mock.Anything).Once()
+	mockWriter.On("Warningf", mock.Anything, mock.Anything, mock.Anything).Once()
 	mockWriter.On("EndIf").Once()
 
 	err := shell.writeScript(mockWriter, common.BuildStageUploadOnSuccessArtifacts, info)
@@ -295,7 +331,7 @@ func TestGitCleanFlags(t *testing.T) {
 			mockWriter := new(MockShellWriter)
 			defer mockWriter.AssertExpectations(t)
 
-			mockWriter.On("Notice", "Checking out %s as %s...", dummySha[0:8], dummyRef).Once()
+			mockWriter.On("Noticef", "Checking out %s as %s...", dummySha[0:8], dummyRef).Once()
 			mockWriter.On("Command", "git", "checkout", "-f", "-q", dummySha).Once()
 
 			if test.expectedGitClean {
@@ -335,7 +371,6 @@ func TestGitFetchFlags(t *testing.T) {
 			const dummySha = "01234567abcdef"
 			const dummyRef = "master"
 			const dummyProjectDir = "./"
-			const dummyGitDir = "./.git"
 
 			build := &common.Build{
 				Runner: &common.RunnerConfig{},
@@ -350,14 +385,15 @@ func TestGitFetchFlags(t *testing.T) {
 			mockWriter := new(MockShellWriter)
 			defer mockWriter.AssertExpectations(t)
 
-			mockWriter.On("Notice", "Fetching changes...").Once()
+			mockWriter.On("Noticef", "Fetching changes...").Once()
 			mockWriter.On("MkTmpDir", mock.Anything).Return(mock.Anything).Once()
 			mockWriter.On("Command", "git", "config", "-f", mock.Anything, "fetch.recurseSubmodules", "false").Once()
 			mockWriter.On("Command", "git", "init", dummyProjectDir, "--template", mock.Anything).Once()
 			mockWriter.On("Cd", mock.Anything)
+			mockWriter.On("Join", mock.Anything, mock.Anything).Return(mock.Anything).Once()
 			mockWriter.On("IfCmd", "git", "remote", "add", "origin", mock.Anything)
 			mockWriter.On("RmFile", mock.Anything)
-			mockWriter.On("Notice", "Created fresh repository.").Once()
+			mockWriter.On("Noticef", "Created fresh repository.").Once()
 			mockWriter.On("Else")
 			mockWriter.On("Command", "git", "remote", "set-url", "origin", mock.Anything)
 			mockWriter.On("EndIf")
@@ -366,7 +402,7 @@ func TestGitFetchFlags(t *testing.T) {
 			command = append(command, test.expectedGitFetchFlags...)
 			mockWriter.On("Command", command...)
 
-			shell.writeRefspecFetchCmd(mockWriter, build, dummyProjectDir, dummyGitDir)
+			shell.writeRefspecFetchCmd(mockWriter, build, dummyProjectDir)
 		})
 	}
 }
@@ -376,7 +412,7 @@ func TestAbstractShell_writeSubmoduleUpdateCmdRecursive(t *testing.T) {
 	mockWriter := new(MockShellWriter)
 	defer mockWriter.AssertExpectations(t)
 
-	mockWriter.On("Notice", "Updating/initializing submodules recursively...").Once()
+	mockWriter.On("Noticef", "Updating/initializing submodules recursively...").Once()
 	mockWriter.On("Command", "git", "submodule", "sync", "--recursive").Once()
 	mockWriter.On("Command", "git", "submodule", "update", "--init", "--recursive").Once()
 	mockWriter.On("Command", "git", "submodule", "foreach", "--recursive", "git clean -ffxd").Once()
@@ -393,7 +429,7 @@ func TestAbstractShell_writeSubmoduleUpdateCmd(t *testing.T) {
 	mockWriter := new(MockShellWriter)
 	defer mockWriter.AssertExpectations(t)
 
-	mockWriter.On("Notice", "Updating/initializing submodules...").Once()
+	mockWriter.On("Noticef", "Updating/initializing submodules...").Once()
 	mockWriter.On("Command", "git", "submodule", "sync").Once()
 	mockWriter.On("Command", "git", "submodule", "update", "--init").Once()
 	mockWriter.On("Command", "git", "submodule", "foreach", "git clean -ffxd").Once()
@@ -435,7 +471,7 @@ func TestWriteUserScript(t *testing.T) {
 			setupExpectations: func(m *MockShellWriter) {
 				m.On("Variable", mock.Anything)
 				m.On("Cd", mock.AnythingOfType("string"))
-				m.On("Notice", "$ %s", "echo hello").Once()
+				m.On("Noticef", "$ %s", "echo hello").Once()
 				m.On("Line", "echo hello").Once()
 				m.On("CheckForErrors").Once()
 			},
@@ -462,9 +498,9 @@ func TestWriteUserScript(t *testing.T) {
 			setupExpectations: func(m *MockShellWriter) {
 				m.On("Variable", mock.Anything)
 				m.On("Cd", mock.AnythingOfType("string"))
-				m.On("Notice", "$ %s", "echo prebuild").Once()
-				m.On("Notice", "$ %s", "echo release").Once()
-				m.On("Notice", "$ %s", "echo postbuild").Once()
+				m.On("Noticef", "$ %s", "echo prebuild").Once()
+				m.On("Noticef", "$ %s", "echo release").Once()
+				m.On("Noticef", "$ %s", "echo postbuild").Once()
 				m.On("Line", "echo prebuild").Once()
 				m.On("Line", "echo release").Once()
 				m.On("Line", "echo postbuild").Once()
@@ -672,8 +708,13 @@ func TestSkipBuildStage(t *testing.T) {
 
 					// empty stages should always be skipped
 					err := shell.writeScript(&BashWriter{}, stage, info)
-					assert.True(t, errors.Is(err, common.ErrSkipBuildStage),
-						"expected err %T, but got %T", common.ErrSkipBuildStage, err)
+					assert.True(
+						t,
+						errors.Is(err, common.ErrSkipBuildStage),
+						"expected err %T, but got %T",
+						common.ErrSkipBuildStage,
+						err,
+					)
 
 					// stages with bare minimum requirements should not be skipped
 					build.JobResponse = tc.JobResponse
